<?php
/********************************* PHP-P10 ******************************
 *    P10 uplink class by pk910   (c)2011 pk910                         *
 ************************************************************************
 *                          Version 2 (OOP)                             *
 *                                                                      *
 * PHP-P10 is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with PHP-P10; if not, write to the Free Software Foundation,   *
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.       *
 *                                                                      *
 ************************************************************************
 * 
 *  Uplink/Client.class.php
 *
 * This file contains the basic Client Socket.
 *
 ************************************************************************
 * accessable methods:
 *
 * bool connect(String $host, int $port, String $bind = null, bool $ssl = false, int $blocking = 0)
 *     connects the socket to $host:$port with the provided options
 *
 * void disconnect()
 *     disconnects the socket
 *
 * bool connected()
 *     returns the state of the socket (true if the socket is connected with the server)
 *
 * String[] recv()
 *     tries to receive datas from the socket. Returns null or a array containing the lines received.
 *     this method BLOCKS THE SCRIPT EXECUTION for the defined time if $blocking is set on connect().
 *
 * void send(String $line, String $newline = "\r\n")
 *     tries to send datas through the socket.
 *
 * Array[] getTraffic()
 *     returns an array containing the traffic the socket has received and sent.
 *     index in = incoming (received) traffic count
       index out = outgoing (sent) traffic count
 */

class Client {
	const CLIENT_RECV_MAX_LINES = 20; //maximum Lines to receive within one recv() call
	
	private $socket;
	private $traffic = array("in" => 0, "out" => 0);
	
	public function connect($host, $port, $bind = null, $ssl = false, $blocking = 0) {
		if($bind)
			$options = array('socket' => array('bindto' => $bind.":0"));
		else
			$options = array();
		$context = stream_context_create($options);
		$sock = stream_socket_client(($ssl ? 'ssl://' : '').$host.':'.$port, $errno, $errstr, 3, STREAM_CLIENT_CONNECT, $context);
		if($sock) {
			if($blocking) {
				stream_set_blocking($sock, true);
				stream_set_timeout($sock, 0, ($blocking / 1000));
			} else {
				stream_set_blocking($sock, false);
			}
			$this->socket = $sock;
			return true;
		} else
			return false;
	}
	
	public function disconnect() {
		if($this->socket == null) return;
		fclose($this->socket);
		$this->socket = null;
	}
	
	public function connected() {
		if($this->socket == null) return false;
		$read = array($this->socket);
		$write= null;
		$except=null;
		$n=@stream_select($read, $write, $except, 0);
		if($n === FALSE || feof($this->socket)) {
			$this->socket = false;
			return false;
		}
		return true;
	}
	
	public function recv() {
		$read = array($this->socket);
		$write= null;
		$except=null;
		$n=@stream_select($read, $write, $except, 0);
		if($n === FALSE || feof($this->socket)) {
			$this->socket = false;
			return null;
		} elseif($n > 0) {
			$lines = array();
			while(($line = @fgets($this->socket)) != null) {
				$line=trim($line);
				if(!empty($line)) {
					echo"[recv] ".$line."\n";
					$this->traffic['in'] += strlen($line);
					$lines[] = $line;
					if(count($lines) >= self::CLIENT_RECV_MAX_LINES) break;
				}
			}
			return $lines;
		}
		return null;
	}
	
	public function send($line, $newline = "\r\n") {
		if($this->socket == null) return;
		echo"[send] ".$line."\n";
		$this->traffic['out'] += strlen($line);
		fwrite($this->socket,$line.$newline);
	}
	
	public function getTraffic() {
		return $this->traffic;
	}
}

?>